%{
/*****************************************************************************
 *
 * FILE:	conf_grammar.y
 * DESCRIPTION:	configuration parser
 * DATE:	Thu, Sep  9 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *  Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: conf_grammar.y,v 1.21.2.2 2002/08/10 20:49:36 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <string.h>
#include <sys/types.h>
#include <pwd.h>
#include <grp.h>
#include "wyvern.h"
#include "functions.h"

#ifndef	HAVE_SETLINEBUF
#define	setlinebuf(x)	setvbuf((x), (char *)NULL, _IOLBF, 0)
#endif	/* HAVE_SETLINEBUF */

struct passwd *	pbuf;
struct group *	gbuf;

static char	mimeType[STD_BUFSIZE];
static char	fakeName[STD_BUFSIZE];
static char	moduleName[TINY_BUFSIZE];
%}

%union {
  int		ival;	/* INTEGER	*/
  char *	sval;	/* STRING	*/
};

/* wyvern.conf tokens	*/
%token W_PORT W_SECUREPORT W_MAXREQUEST
%token W_USER W_GROUP W_SERVERADMIN W_SERVERROOT
%token W_ERRORLOG W_ACCESSLOG
%token W_PIDFILE W_SERVERNAME W_TIMEOUT

%token W_DOCUMENTROOT W_USERDIR W_DIRECTORYINDEX
%token W_FOLLOWSYMLINKS W_DEFAULTTYPE W_ERROR_DOCUMENT W_ADDTYPE
%token W_REDIRECT W_ALIAS W_SCRIPTALIAS W_MODULE W_BINDADDRESS

%token W_BEGIN_VIRTUALHOST W_END_VIRTUALHOST

%token <ival> INTEGER
%token <sval> STRING
%token <sval> MIMETYPE

%%

ConfigFiles:		WyvernConfigStmts MimeTypeStmts
	;

WyvernConfigStmts:	WyvernConfigStmts WyvernConfigStmt
	|		/* empty */
	;

WyvernConfigStmt:
			PortStmt
	|		MaxRequestStmt
	|		TimeoutStmt
	|		UserStmt
	|		GroupStmt
	|		PidFileStmt
	|		ServerRootStmt
	|		ErrorLogStmt
	|		AccessLogStmt
	|		ServerAdminStmt
	|		ServerNameStmt
	|		DocumentRootStmt
	|		UserDirStmt
	|		DirectoryIndexStmt
	|		FollowSymLinksStmt
	|		DefaultTypeStmt
	|		ErrorDocumentStmt
	|		AddTypeStmt
	|		RedirectStmt
	|		AliasStmt
	|		ScriptAliasStmt
	|		ModuleStmt
	|		BindAddressStmt
	|		BeginVirtualHostStmt
	|		EndVirtualHostStmt
	;

PortStmt:		W_PORT INTEGER
			{
			  httpd->port = $2;
			}
	|		W_SECUREPORT INTEGER
			{
#if	USE_SSL
			  httpd->portSSL = $2;
#endif	/* USE_SSL */
			}
	;

MaxRequestStmt:		W_MAXREQUEST INTEGER
			{
			  httpd->maxRequest = ($2 <= 1 ? DEF_MIN_REQUEST : $2);
			}
	;

TimeoutStmt:		W_TIMEOUT INTEGER
			{
			  if ($2 < 0) {
			    fputs("Timeout is positive value\n", stderr);
			    exit(1);
			  }
			  httpd->timeout = $2;
			}
	;

UserStmt:		W_USER STRING
			{
			  pbuf = getpwnam($2);
			  if (pbuf == NULL) {
			    fprintf(stderr, "No such user: %s\n", $2);
			    exit(1);
			  }
			  httpd->uid = pbuf->pw_uid;
			}
	|		W_USER INTEGER
			{
			  pbuf = getpwuid((uid_t)$2);
			  if (pbuf == NULL) {
			    fprintf(stderr, "No such user ID: %d\n", $2);
			    exit(1);
			  }
			  httpd->uid = pbuf->pw_uid;
			}
	;

GroupStmt:		W_GROUP STRING
			{
			  gbuf = getgrnam($2);
			  if (gbuf == NULL) {
			    fprintf(stderr, "No such group: %s\n", $2);
			    exit(1);
			  }
			  httpd->gid = gbuf->gr_gid;
			}
	|		W_GROUP INTEGER
			{
			  gbuf = getgrgid((gid_t)$2);
			  if (gbuf == NULL) {
			    fprintf(stderr, "No such group ID: %d\n", $2);
			    exit(1);
			  }
			  httpd->gid = gbuf->gr_gid;
			}
	;

PidFileStmt:		W_PIDFILE STRING
			{
			  httpd->pidFile = xstrdup($2);
			}
	;

ServerRootStmt:		W_SERVERROOT STRING
			{
			  httpd->serverRoot = xstrdup($2);
			  chdir(httpd->serverRoot);	/* for mime.types */
			}
	;

ErrorLogStmt:		W_ERRORLOG STRING
			{
			  if ($2 != NULL) {
			    httpd->vhost->errorLog = fopen($2, "a");
			    if (httpd->vhost->errorLog == NULL) {
			      die("failed to open errorLog: %s", $2);
			    }
			    (void)setlinebuf(httpd->vhost->errorLog);
			  }
			}
	;

AccessLogStmt:		W_ACCESSLOG STRING
			{
			  if ($2 != NULL) {
			    httpd->vhost->accessLog = fopen($2, "a");
			    if (httpd->vhost->accessLog == NULL) {
			      die("failed to open accessLog: %s", $2);
			    }
			    (void)setlinebuf(httpd->vhost->accessLog);
			  }
			}
	;

ServerAdminStmt:	W_SERVERADMIN STRING
			{
			  httpd->vhost->serverAdmin = xstrdup($2);
			}
	;

ServerNameStmt:		W_SERVERNAME STRING
			{
			  httpd->vhost->serverName = xstrdup($2);
			}
	;

DocumentRootStmt:	W_DOCUMENTROOT STRING
			{
			  httpd->vhost->docRoot = xstrdup($2);
			}
	;

UserDirStmt:		W_USERDIR STRING
			{
			  httpd->vhost->userDir = xstrdup($2);
			}
	;

DirectoryIndexStmt:	W_DIRECTORYINDEX STRING
			{
			  httpd->vhost->dirIndex = xstrdup($2);
			}
	;

FollowSymLinksStmt:	W_FOLLOWSYMLINKS STRING
			{
			  if (strcasecmp($2, "on") == 0) {
			    httpd->vhost->symlinks = TRUE;
			  }
			  else {
			    httpd->vhost->symlinks = FALSE;
			  }
			}
	;

DefaultTypeStmt:	W_DEFAULTTYPE STRING
			{
			  httpd->vhost->defaultType = xstrdup($2);
			}
	;

ErrorDocumentStmt:	W_ERROR_DOCUMENT INTEGER STRING
			{
			  registerErrorDocument($2, $3);
			}
	;

AddTypeStmt:		W_ADDTYPE MimeTypeStmt
	;

RedirectStmt:		W_REDIRECT STRING
			{
			  memset(fakeName, 0, sizeof(fakeName));
			  xstrncpy(fakeName, $2, sizeof(fakeName));
			}
			STRING
			{
			  registerAliases(fakeName, $4, A_REDIRECT);
			}
	;

AliasStmt:		W_ALIAS STRING
			{
			  memset(fakeName, 0, sizeof(fakeName));
			  xstrncpy(fakeName, $2, sizeof(fakeName));
			}
			STRING
			{
			  registerAliases(fakeName, $4, A_ALIAS);
			}
	;

ScriptAliasStmt:	W_SCRIPTALIAS STRING
			{
			  memset(fakeName, 0, sizeof(fakeName));
			  xstrncpy(fakeName, $2, sizeof(fakeName));
			}
			STRING
			{
			  registerAliases(fakeName, $4, A_SCRIPT);
			}
	;

ModuleStmt:		W_MODULE STRING
			{
			  memset(moduleName, 0, sizeof(moduleName));
			  xstrncpy(moduleName, $2, sizeof(moduleName));
			}
			STRING
			{
			  loadModule(moduleName, $4);
			}
	;

BindAddressStmt:	W_BINDADDRESS STRING
			{
			  if ($2[0] != '*') {
			    httpd->bindAddress = xstrdup($2);
			  }
			}
	;

BeginVirtualHostStmt:	W_BEGIN_VIRTUALHOST
			{
			  httpd->vhostList = newVirtualHost(httpd->vhostList);
			  httpd->vhost	   = httpd->vhostList;
			  httpd->vhost->serverName = xstrdup(yylval.sval);
			}
	;

EndVirtualHostStmt:	W_END_VIRTUALHOST
			{
			  httpd->vhost = httpd->host;
			}
	;

/*
 *	mime.types
 */

MimeTypeStmts:		MimeTypeStmts MimeTypeStmt
	|		/* empty */
	;

MimeTypeStmt:		MIMETYPE
			{
			  register char *	p = $1;
			  if (*p == '"') {
			    p++;
			    p[strlen(p) - 1] = '\0';
			  }
			  memset(mimeType, 0, sizeof(mimeType));
			  xstrncpy(mimeType, p, sizeof(mimeType));
			}
			SuffixList
	;

SuffixList:		SuffixList Suffix
	|		/* empty */
	;

Suffix:			STRING
			{
			  registerMimeType($1, mimeType);
			}
	;

%%
