/*****************************************************************************
 *
 * FILE:	daemon.c
 * DESCRIPTION:	make a daemon
 * DATE:	Thu, Sep 14 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: daemon.c,v 1.15.2.3 2002/01/29 10:24:25 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/stat.h>
#include <pwd.h>
#include <grp.h>
#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	detach
 * Description:		detach the controlling terminal and create a new
 *			session
 * Arguments:		none
 * Return Value:	none
 */
void
detach(void)
{
  /* detach the current process from shell */
  switch (fork()) {
    case -1:	/* error */
      perror("fork");
      exit(1);
    case  0:	/*  child process */
      break;
    default:	/* parent process */
      _exit(0);
  }

  /* create new session */
  if (setsid() == -1) {
    perror("setsid");
    exit(1);
  }

  /* detach the process from the 1st child process */
  switch (fork()) {
    case -1:	/* error */
      perror("fork");
      exit(1);
    case  0:	/*  child process */
      break;
    default:	/* parent process */
      _exit(0);
  }

  /* change working directory */
  (void)chdir("/");

  /* clear the file mode mask */
  (void)umask(0022);

  if (freopen("/dev/null", "r", stdin) == NULL) {
    /* ignore error */
  }
  if (freopen("/dev/null", "w", stdout) == NULL) {
    /* ignore error */
  }
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	storePID
 * Description:		store the PID to the pid_file
 * Arguments:		none
 * Return Value:	none
 */
void
storePID(void)
{
  FILE *	fout;
  struct stat	sbuf;
  static pid_t	saved_pid = -1;

  /* get current pid */
  httpd->pid = getpid();

  /* check duplex boot with same wyvern.config */
  if (httpd->pid != saved_pid && lstat(httpd->pidFile, &sbuf) == 0) {
    wvSyslog(LOG_WARNING, "pid file overwritten -- previous Wyvern run?\n");
  }

  /* store wyvern pid */
  fout = fopen(httpd->pidFile, "w");
  if (fout == NULL) {
    die("failed to open %s\n", httpd->pidFile);
  }
  fprintf(fout, "%ld\n", (long)httpd->pid);
  /* change owner/group of process ID file */
  (void)fchown(fileno(fout), httpd->uid, httpd->gid);	/* ignore error */
  fclose(fout);
  saved_pid = httpd->pid;
}

/*
 * Function Name:	setOwner
 * Description:		change owner and group of wyvern process
 * Arguments:		none
 * Return Value:	none
 */
void
setOwner(void)
{
  /* set uid/gid */
  if (getuid() == 0) {	/* I am a Super User! */
    struct passwd *	pwd;

    pwd = getpwuid(httpd->uid);
    if (pwd != NULL &&
	initgroups(pwd->pw_name, (int)pwd->pw_gid) != -1 &&
	setgid(httpd->gid) != -1 &&
	setuid(httpd->uid) != -1) {
      return;	/* successful */
    }
    die("failed to change uid %d and gid %d\n", httpd->uid, httpd->gid);
  }
  else {	/* I am an user... */
    wvSyslog(LOG_WARNING,
	     "NO RUN AS ROOT: failed to change uid %d and gid %d\n",
	     httpd->uid, httpd->gid);
    httpd->uid = getuid();
    httpd->gid = getgid();
  }
}

/*
 * Function Name:	setSystemResource
 * Description:		limit system resource of wyvern process
 * Arguments:		none
 * Return Value:	none
 */
void
setSystemResource(void)
{
  struct rlimit	rlp;

  if (getrlimit(RLIMIT_CORE, &rlp) == 0) {
    rlp.rlim_cur = 0;
    setrlimit(RLIMIT_CORE, &rlp);
  }
#if	0
  if (getrlimit(RLIMIT_FSIZE, &rlp) == 0) {
    rlp.rlim_cur = MAX_FILE_SIZE;
    setrlimit(RLIMIT_FSIZE, &rlp);
  }
#endif
  if (getrlimit(RLIMIT_NOFILE, &rlp) == 0) {
    rlp.rlim_cur = 1024;
    setrlimit(RLIMIT_NOFILE, &rlp);
  }
}

