/*****************************************************************************
 *
 * FILE:	functions.h
 * DESCRIPTION:	functions declaration file for wyvern sources
 * DATE:	Mon, Sep 27 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: functions.h,v 1.47.2.5 2002/01/30 13:26:33 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_functions_h
#define	_functions_h


/*
 *	alias.c
 */
extern void	registerAliases(String, String, alias_t);
extern bool_t	parseURI(Thread *);


/*
 *	alloc.c
 */
extern void	cleanupMemoryBlock(void);
extern void *	xmalloc(size_t);
extern void	xfree(void *);
extern void *	xcalloc(size_t, size_t);
extern void *	xrealloc(void *, size_t);
#if	0
extern void *	xreallocf(void *, size_t);
#endif
extern char *	xstrdup(const char *);
extern char *	xstrndup(const char *, size_t);
#if __STDC__
extern int	xasprintf(char **, const char *, ...);
#else
extern int	xasprintf(str, fmt, va_alist);
#endif	/* __STDC__ */


/*
 *	cond.c
 */
extern void	checkConditionalRequest(Thread *);


/*
 *	config.c
 */
extern void	readConfigFile(void);
extern void	Reconfiguration(void);


/*
 *	daemon.c
 */
extern void	detach(void);
extern void	storePID(void);
extern void	setOwner(void);
extern void	setSystemResource(void);


/*
 *	date.c
 */
extern String	getTimeOfRFC822(time_t);
extern time_t	convertDate(String);


/*
 *	error.c
 */
extern void	initErrorDocumentTable(void);
extern void	freeErrorDocumentTable(void);
extern void	registerErrorDocument(int, String);
extern String	getErrorDocument(int);
extern void	sendErrorMessage(Thread *);


/*
 *	get.c
 */
void	openRequestFile(Thread *);


/*
 *	io.c
 */
extern int	nread(Thread *, String, size_t);
extern int	readline(Thread *, String, size_t);
extern int	iosFlush(Thread *);
extern int	iosWrite(Thread *, const char *, size_t);
extern int	iosPuts(Thread *, String);
extern int	iosPrintf(Thread *, const char *, ...);


/*
 *	log.c
 */
extern void	openLogFiles(void);
extern void	closeLogFiles(void);
extern void	writeLogs(Thread *);
extern void	die(const char *, ...);
#ifndef	USE_SYSLOG
extern void	wvSyslog(log_t, const char *, ...);
#endif	/* USE_SYSLOG */


/*
 *	mime.c
 */
extern void	initMimeTypeTable(void);
extern void	freeMimeTypeTable(void);
extern void	registerMimeType(String, String);
extern void	setMimeType(Thread *);
#if	DEBUG
extern void	debugMimeType(void);
#endif	/* DEBUG */


/*
 *	module.c
 */
extern void	initPluggableModules(void);
extern void	loadModule(String, String);
extern void	cleanupModules(void);


/*
 *	network.c
 */
extern void	openServiceChannel(unsigned int);
extern int	xclose(int);
extern String	getServerName(void);
#if	USE_IPV6
extern void	setClientName(Thread *, struct sockaddr_storage *);
#else
extern void	setClientName(Thread *, struct sockaddr_in *);
#endif	/* USE_IPV6 */
extern void	sendMail(void);


/*
 *	request.c
 */
extern void	foreverRunning(void);
extern void	allowRequest(Thread *);
extern String	getAnyHeaderField(String);


/*
 *	response.c
 */
extern void	sendResponse(Thread *);
extern void	sendingEntity(Thread *);
extern void	setETag(Thread *);


/*
 *	script.c
 */
extern void	initServerEnvironment(void);
extern void	freeServerEnvironment(void);
extern bool_t	setScriptEnvironment(Thread *);
#if	DEBUG
extern void	debugServerEnvironment(void);
#endif	/* DEBUG */


/*
 *	signal.c
 */
extern void	setSignalTrap(void);


/*
 *	status.c
 */
extern String	getResponsePhrase(rsCode_t);
extern int	getResponseCode(rsCode_t);
extern void	setResponseCode(Thread *, rsCode_t);


#if	USE_SSL
/*
 *	ssl.c
 */
extern void	sslOpenServiceChannel(void);
extern void	sslCleanup(void);
extern int	sslAcceptNewConnection(Thread *);
extern ssl_t	sslEstablishConnection(Thread *);
#endif	/* USE_SSL */


/*
 *	thread.c
 */
extern void	initThreadQueue(void);
extern void	freeThreadQueue(void);
extern void	flushThreadQueue(void);
extern Thread *	newThread(int);
extern void	detachThread(Thread *);
extern void	cleanupThread(Thread *);
extern Thread *	getThreadHead(void);
extern void	sweepThreadQueue(void);


/*
 *	user.c
 */
extern String	getHomeDir(String);
extern void	initUserCache(void);
extern void	freeUserCache(void);


/*
 *	util.c
 */
extern char *	itoa(long int);
extern char *	itoxa(unsigned long int);
extern int	checkSymlinkFile(String);
extern bool_t	skipLine(String);
extern String	chomp(String);
extern String	decodeEscapedURI(String);
extern String	xstrncpy(String, const String, size_t);
#ifndef	HAVE_USLEEP
extern int	usleep(unsigned int);
#endif	/* HAVE_USLEEP */


/*
 *	vhost.c
 */
extern VirtualHost *	newVirtualHost(VirtualHost *);
extern void	 	deleteVirtualHosts(void);
extern void		verifyVirtualHosts(void);
extern bool_t		switchVirtualHost(Thread *);


/*
 *	wyvern.c
 */
extern void	quit(void);


#endif	/* _functions_h */
