/*****************************************************************************
 *
 * FILE:	auth.c
 * DESCRIPTION:	authentication functions
 * DATE:	Mon, Sep 18 2000
 * UPDATE:	Sun, Dec 19 2004
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(c) 2000-2004 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *  Copyright (c) 2000-2004 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: auth.c,v 1.21.2.5 2004/12/19 13:52:12 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <string.h>
#if	!defined(USE_SSL)
#include "md5.h"
#endif	/* USE_SSL, USE_CONTENT_MD5 */
#include "wyvern.h"
#include "wdbm.h"
#include "auth.h"
#include "base64.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static bool_t	compareAddr(String, String);


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	readWPasswd
 * Description:		read '.wpasswd' file to authenticate
 * Arguments:		wpasswd structure, file path of '.wpasswd'
 * Return Value:	successful: 0, otherwise: -1
 */
int
readWPasswd(wpasswd)
	register struct _wpasswd *	wpasswd;
{
  WDBM_FILE		dp;
  wdatum		key;	/* username */
  wdatum		data;	/* password */
  struct stat		sbuf;

  if (lstat(wpasswd->path, &sbuf) == -1) {
    wpasswd->mtime = (time_t)0;
  }
  else {
    wpasswd->mtime = sbuf.st_mtime;
  }
  wpasswd->authMgr = NULL;

  /* open '.wpasswd.db' file */
  dp = wdbm_open(wpasswd->path, WDBM_READER, 0400);
  if (dp == NULL) {
    return -1;
  }

  for (key = wdbm_firstkey(dp); key.dptr; key = wdbm_nextkey(dp, key)) {
    AuthMgr *	ap;

    data = wdbm_fetch(dp, key);
    if (data.dptr == NULL) {
      continue;	/* ignore error ... */
    }

    ap = (AuthMgr *)xcalloc(1, sizeof(AuthMgr));
    if (ap == NULL) {
      continue;	/* ignore error... */
    }
    ap->username	= xstrndup(key.dptr,  (size_t)key.dsize);
    ap->password	= xstrndup(data.dptr, (size_t)data.dsize);
    ap->next		= wpasswd->authMgr;
    wpasswd->authMgr	= ap;
#if	DEBUG
    fprintf(stderr, "[AUTH]: %s = %s\n", ap->username, ap->password);
#endif	/* DEBUG */
  }
  wdbm_close(dp);

  return 0;
}

/*
 * Function Name:	readWAccess
 * Description:		read '.waccess' file to authenticate
 * Arguments:		waccess structure, file path for '.waccess'
 * Return Value:	successful: 0, otherwise: -1
 */
int
readWAccess(waccess)
	register struct _waccess *	waccess;
{
#define	MAX_ARGS	3
#define	A_TYPE		0
#define	A_VALUE		1
#define	A_REST		2
  register FILE *	fin;
  char			buf[BUF_SIZE];
  struct stat		sbuf;

  if (lstat(waccess->path, &sbuf) == -1) {
    waccess->mtime = (time_t)0;
  }
  else {
    waccess->mtime = sbuf.st_mtime;
  }
  waccess->authDir = NULL;

  /* open '.waccess' file */
  fin = fopen(waccess->path, "r");
  if (fin == NULL) {
    return -1;
  }

  memset(buf, 0, sizeof(buf));
  while (fgets(buf, sizeof(buf), fin) != NULL) {
    String *	sp;
    String	args[MAX_ARGS];
    String	p = buf;
    int		n = 0;

    if (skipLine(buf)) {	/* skip comment lines */
      continue;
    }

    for (sp = args; (*sp = strsep(&p, " \t\r\n")) != NULL; ) {
      if (**sp != EOL) {
	n++;
	if (++sp >= &args[MAX_ARGS]) {
	  break;
	}
      }
    }

    if (strcasecmp(args[A_TYPE], "AuthDir") == 0) {
      AuthDir *	ap = (AuthDir *)xcalloc(1, sizeof(AuthDir));
      if (ap == NULL) {
	return -1;
      }
      ap->dir		= xstrdup(args[A_VALUE]);
      ap->allow		= NULL;
      ap->next		= waccess->authDir;
      waccess->authDir	= ap;
    }
    else if (strcasecmp(args[A_TYPE], "AuthUser") == 0) {
      Allow * allow = (Allow *)xcalloc(1, sizeof(Allow));
      if (allow == NULL) {
	return -1;	/* may cause memory leak...*/
      }
      if (n == 3 && strcasecmp(args[A_REST], "Digest") == 0) {
	allow->method	= AUTH_DIGEST;
      }
      else {
	allow->method	= AUTH_BASIC;
      }
      allow->type		= AUTH_USER;
      allow->auth.user		= xstrdup(args[A_VALUE]);
      allow->next		= waccess->authDir->allow;
      waccess->authDir->allow	= allow;
    }
    else if (strcasecmp(args[A_TYPE], "AuthAddr") == 0) {
      Allow * allow = (Allow *)xcalloc(1, sizeof(Allow));
      if (allow == NULL) {
	return -1;	/* may cause memory leak...*/
      }
      allow->type		= AUTH_ADDR;
      allow->auth.addr		= xstrdup(args[A_VALUE]);
      allow->next		= waccess->authDir->allow;
      waccess->authDir->allow	= allow;
    }
    else {	/* illegal type */
      wvSyslog(LOG_WARNING, "Illegal authentication type: %s\n", args[A_TYPE]);
    }
    memset(buf, 0, sizeof(buf));
  }
  fclose(fin);

  return 0;
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	checkPassword
 * Description:		check password of user to access
 * Arguments:		thread, AuthMgr
 * Return Value:	allow: TRUE, otherwise: FALSE
 */
bool_t
checkPassword(thd, auth)
	register Thread *	thd;
	AuthMgr *		auth;
{
  String	scheme;
  String	param;

  if (thd->request.authorization == NULL) {
    return FALSE;
  }

  /*
   * divide 'authorization' into 'scheme' and 'param'
   */
  scheme = thd->request.authorization;
  param	 = (String)strchr(thd->request.authorization, SPC);
  if (param == NULL) {
    return FALSE;
  }
  *param++ = EOL;

  /*
   * basic authentication scheme
   */
  if (strncasecmp(scheme, "BASIC", 5) == 0) {
#define	MAX_AUTH_ARGS	2
#define	A_USERNAME	0
#define	A_PASSWORD	1
    String *		sp;
    String		args[MAX_AUTH_ARGS];
    String		p = decodeBase64(param);
    register AuthMgr *	ap;

    /* set username and password */
    for (sp = args; (*sp = strsep(&p, ":")) != NULL; ) {
      if (**sp != EOL) {
	if (++sp >= &args[MAX_AUTH_ARGS]) {
	  break;
	}
      }
    }

    for (ap = auth; ap != NULL; ap = ap->next) {
      if (strcmp(args[A_USERNAME], ap->username) == 0) {
	extern void	MD5Init(MD5_CTX *);
	extern void	MD5Update(MD5_CTX *, unsigned char *, unsigned int);
	extern void	MD5Final(unsigned char [MD5_DIGEST_LENGTH], MD5_CTX *);

	MD5_CTX		context;
	size_t		len = strlen(args[A_PASSWORD]);
	int		i;
	unsigned char	md5[TINY_BUFSIZE+1];
	unsigned char	digest[MD5_DIGEST_LENGTH];

	MD5Init(&context);
	MD5Update(&context, args[A_PASSWORD], len);
	MD5Final(digest, &context);

	memset(md5, 0, sizeof(md5));
	for (i = 0; i < MD5_DIGEST_LENGTH; i++) {
	  (void)snprintf(md5 + i * 2, sizeof(md5), "%02x", digest[i]);
	}
#if	DEBUG
	fprintf(stderr, "[auth-in]: %s:%s\n", args[A_PASSWORD], md5);
	fprintf(stderr, "[auth-db]: %s:%s\n", ap->username, ap->password);
#endif	/* DEBUG */
	if (strcmp(ap->password, md5) == 0) {
	  thd->authUser = xstrdup(ap->username);
	  return TRUE;
	}
	return FALSE;
      }
    } /* end of for */
  }
#if	SOMEDAY_IMPLEMENTED
  /*
   * digest authentication scheme
   */
  else if (strncasecmp(scheme, "DIGEST", 6) == 0) {
  }
  /*
   * unknown authentication scheme
   */
  else {
    setResponseCode(thd, CODE_UNAUTHORIZED);
  }
#endif	/* SOMEDAY_IMPLEMENTED */

  return FALSE;
}

/*
 * Function Name:	checkAddress
 * Description:		check IP address/domain name to access
 * Arguments:		thread, Allow
 * Return Value:	allow: TRUE, otherwise: FALSE
 */
bool_t
checkAddress(thd, allow)
	register Thread *	thd;
	Allow *			allow;
{
  register Allow *	ap;

  for (ap = allow; ap != NULL; ap = ap->next) {
    if (ap->type == AUTH_USER) {
      continue;
    }
    if ((compareAddr(thd->client.name, ap->auth.addr) == TRUE) ||
	(compareAddr(thd->client.ip,   ap->auth.addr) == TRUE)) {
      return TRUE;
    }
  }

  return FALSE;
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	compareAddr
 * Description:		check the hostname, IP addr and the access list
 * Arguments:		Hostname/IP addr, compared one
 * Return Value:	Matching: TRUE, Otherwise: FALSE
 */
static bool_t
compareAddr(name, pat)
	String	name;
	String	pat;
{
  size_t	nlen;
  size_t	plen;

  if (name == NULL || pat == NULL) {
    return FALSE;
  }
  nlen = strlen(name);
  plen = strlen(pat);

  /*
   * check to match the name with the pattern like as '.ad.jp'
   */
  if (pat[0] == '.') {
    if (nlen >= plen) {
      size_t	offset = nlen - plen;
      if (strncmp(name + offset, pat, plen) == 0) {
	return TRUE;
      }
    }
  }
  /*
   * check to match the name with the pattern like as '133.24.' or IP addr
   */
  else if (pat[0] >= '0' && pat[0] <= '9') {
    if (strncmp(name, pat, plen) == 0) {
      return TRUE;	/* match! */
    }
  }
  /*
   * check to match the name with the pattern of FQDN 
   */
  else {
    if (strcmp(name, pat) == 0) {
      return TRUE;	/* match! */
    }
  }

  return FALSE;	/* no match */
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	freeAuthMgrList
 * Description:		release memory area of AuthMgr
 * Arguments:		pointer of AuthMgr
 * Return Value:	none
 */
void
freeAuthMgrList(auth)
	AuthMgr *	auth;
{
  register AuthMgr *	ap;
  register AuthMgr *	tmp;

  for (ap = auth; ap != NULL; ap = tmp) {
    tmp = ap->next;
    xfree((String)ap->username);
    xfree((String)ap->password);
    xfree((AuthMgr *)ap);
  }
}

/*
 * Function Name:	freeAllowList
 * Description:		release memory area of Allow
 * Arguments:		pointer of Allow
 * Return Value:	none
 */
void
freeAllowList(allow)
	Allow *	allow;
{
  register Allow *	ap;
  register Allow *	tmp;

  for (ap = allow; ap != NULL; ap = tmp) {
    tmp = ap->next;
    if (ap->type == AUTH_USER) {
      xfree((String)ap->auth.user);
    }
    else {
      xfree((String)ap->auth.addr);
    }
    xfree((Allow *)ap);
  }
}

/*
 * Function Name:	freeAuthDirList
 * Description:		release memory area of AuthDir
 * Arguments:		pointer of AuthDir
 * Return Value:	none
 */
void
freeAuthDirList(authDir)
	AuthDir *	authDir;
{
  register AuthDir *	ap;
  register AuthDir *	tmp;

  for (ap = authDir; ap != NULL; ap = tmp) {
    tmp = ap->next;
    xfree((String)ap->dir);
    freeAllowList(ap->allow);
    xfree((AuthDir *)ap);
  }
}

