/*****************************************************************************
 *
 * FILE:	auth.h
 * DESCRIPTION:	authentication header
 * DATE:	Sat, Sep 18 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *  Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: auth.h,v 1.12 2002/01/18 22:07:47 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_auth_h
#define	_auth_h

#include <sys/stat.h>


/*****************************************************************************
 *
 *	definition of macros and structure
 *
 *****************************************************************************/
#define	SYSTEM_AUTHNAME		"wyvern"

#define	ALLOW_USERS_TABLE	11

#define	WP_NAME_SIZE		10
#define	WP_PASSWD_SIZE		(32+1)


typedef enum {	/* authentication method type */
  AUTH_BASIC,
  AUTH_DIGEST
} auth_method_t;

typedef struct _AuthMgr {	/* manage user name and password */
  String		username;	/* accessible user name */
  String		password;	/* user password	*/
  struct _AuthMgr *	next;
} AuthMgr;

typedef enum {	/* authentication type */
  AUTH_USER,
  AUTH_ADDR
} auth_t;

typedef struct _Allow {	/* manage allowed user name and IP address */
  auth_t		type;
  union {
    String		user;	/* allowed user name */
    String		addr;	/* allowed IP address or domain name */
  } auth;
  auth_method_t		method;
  struct _Allow *	next;
} Allow;

typedef struct _AuthDir {	/* manage authentication directory */
  String		dir;	/* restricted directory name	*/
  Allow *		allow;
  struct _AuthDir *	next;
} AuthDir;

typedef struct _AuthTable {	/* manage authentication */
  String		name;	/* user name */
  struct _waccess {
    String		path;	/* file path */
    time_t		mtime;	/* last data modification */
    AuthDir *		authDir;
  } waccess;
  struct _wpasswd {
    String		path;	/* file path */
    time_t		mtime;	/* last data modification */
    AuthMgr *		authMgr;
  } wpasswd;
  struct _AuthTable *	next;
} AuthTable;


/*****************************************************************************
 *****************************************************************************/

extern int	readWPasswd(struct _wpasswd *);
extern int	readWAccess(struct _waccess *);
extern bool_t	checkPassword(Thread *, AuthMgr *);
extern bool_t	checkAddress(Thread *, Allow *);

extern void	freeAuthMgrList(AuthMgr *);
extern void	freeAllowList(Allow *);
extern void	freeAuthDirList(AuthDir *);

#endif	/* _auth_h */
