/*****************************************************************************
 *
 * FILE:	base64.c
 * DESCRIPTION:	base64 encode/decode program
 * DATE:	Thu, Aug 26 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * REFERENCE:	RFC2045
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *  Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: base64.c,v 1.6.2.3 2002/02/05 12:16:41 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdlib.h>
#include <string.h>
#include "wyvern.h"
#include "functions.h"
#include "base64.h"


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/

/*
 * bit sequence for mask
 *
 * 0xfc = 11111100
 * 0x03 = 00000011
 * 0xf0 = 11110000
 * 0x0f = 00001111
 * 0xc0 = 11000000
 * 0x3f = 00111111
 * 0x30 = 00110000
 * 0x3c = 00111100
 */

/*
 * Base64 alphabet
 */
const char	base64[] =
	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	encodeBase64
 * Description:		encode a string to a base64 string
 * Arguments:		string
 * Return Value:	encoded string, error: NULL
 */
String
encodeBase64(str)
	register String	str;
{
  static String	estr = NULL;
  size_t	len;
  int		i, j;

  len = strlen(str);
  xfree((String)estr);
  estr = (String)xcalloc((size_t)((double)len * 1.5), sizeof(char));
  if (estr == NULL) {
    return NULL;
  }

  /*
   * encode 3-bytes (24-bits) at a time
   */
  for (i = j = 0; i < len - (len % 3); i += 3, j += 4) {
    estr[j]   = base64[(str[i]	  & 0xfc) >> 2];
    estr[j+1] = base64[((str[i]   & 0x03) << 4) | ((str[i+1] & 0xf0) >> 4)];
    estr[j+2] = base64[((str[i+1] & 0x0f) << 2) | ((str[i+2] & 0xc0) >> 6)];
    estr[j+3] = base64[(str[i+2]  & 0x3f)];
  }

  i = len - (len % 3);	/* rest size */
  switch (len % 3) {
    case 2:	/* one character padding */
      estr[j]	= base64[(str[i]   & 0xfc) >> 2];
      estr[j+1] = base64[((str[i]  & 0x03) << 4) | ((str[i+1] & 0xf0) >> 4)];
      estr[j+2] = base64[(str[i+1] & 0x0f) << 2];
      estr[j+3] = base64[64];	/* Pad	*/
      break;
    case 1:	/* two character padding */
      estr[j]	= base64[(str[i] & 0xfc) >> 2];
      estr[j+1]	= base64[(str[i] & 0x03) << 4];
      estr[j+2] = base64[64];	/* Pad	*/
      estr[j+3] = base64[64];	/* Pad	*/
      break;
  }
  estr[j+4] = EOL;

  return estr;
}

/*
 * Function Name:	decodeBase64
 * Description:		decode a base64 string
 * Arguments:		encoded string
 * Return Value:	unencoded string, error: NULL
 */
String
decodeBase64(str)
	register String	str;
{
#define	VAL(x)	(str[(x)] == '=' ? 0 : strchr(base64, str[(x)]) - base64)
  static String	dstr = NULL;
  size_t	len;
  int		i, j;

  len = strlen(str);
  xfree((String)dstr);
  dstr = (String)xcalloc((size_t)len, sizeof(char));
  if (dstr == NULL) {
    return NULL;
  }

  /*
   * work on 3-bytes (24-bits) at a time
   */
  for (i = j = 0; i < len; i += 4, j += 3) {
    dstr[j]   = (VAL(i) << 2) | ((VAL(i+1) & 0x30) >> 4);
    dstr[j+1] = ((VAL(i+1) & 0x0f) << 4) | ((VAL(i+2) & 0x3c) >> 2); 
    dstr[j+2] = ((VAL(i+2) & 0x03) << 6) | (VAL(i+3) & 0x3f);
  }
  dstr[j] = EOL;

  return dstr;
}

