/*****************************************************************************
 *
 * FILE:	mod_morelog.c
 * DESCRIPTION:	wyvern another logging module
 * DATE:	Fri, Sep 15 2000
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2000-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *  Copyright (C) 2000-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_morelog.c,v 1.14.2.1 2002/02/19 11:50:12 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>
#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	declaration of global functions
 *
 *****************************************************************************/
extern void	writeMoreLogs(Thread *);


/*****************************************************************************
 *
 *	definition of macros
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

#if	DEBUG
void
_init(void)
{
  fputs("[mod_morelog]: plugged-in\n", stderr);
}
#endif	/* DEBUG */

/*
 * Function Name:	writeMoreLogs
 * Description:		write the log message in log file
 * Arguments:		thread
 * Return Value:	none
 */
void
writeMoreLogs(thd)
	register Thread *	thd;
{
  /* Informational: 1xx	*/
  /* Successful:    2xx	*/
  /* Redirection:   3xx	*/
  if (thd->rsCode < CODE_BAD_REQUEST) {
    if (thd->request.x_forwarded_for != NULL) {
      fprintf(httpd->accessLog,
	      "%s(%s) ",
	      thd->client.name == NULL ? thd->client.ip : thd->client.name,
	      thd->request.x_forwarded_for);
    }
    else {
      fprintf(httpd->accessLog, "%s ",
	      thd->client.name == NULL ? thd->client.ip : thd->client.name);
    }
    fprintf(httpd->accessLog, "%s %s [%s] \"%s %s %s\" %d ",
		thd->identUser == NULL ? "-" : thd->identUser,
		thd->authUser == NULL ? "-" : thd->authUser,
		getTimeOfRFC822(thd->atime),
		thd->method.name,
		thd->requestURI,
		thd->http.protocol,
		getResponseCode(thd->rsCode));
    if (thd->rsCode > CODE_PARTIAL_CONTENT) {	/* for Redirection: 3xx */
      fputs("- ", httpd->accessLog);
    }
    else {
      if (thd->method.type == M_HEAD || thd->method.type == M_GET) {
#if	HAVE_ZLIB_H
	fprintf(httpd->accessLog, "%ld (%4.1f%%) ",
		(long)thd->file.st_size, thd->gzippct);
#else
	fprintf(httpd->accessLog, "%ld ", (long)thd->file.st_size);
#endif	/* HAVE_ZLIB_H */
      }
      else if (thd->method.type == M_POST) {
	fprintf(httpd->accessLog, "%d ", thd->request.content_length);
      }
      else {
	fputs("- ", httpd->accessLog);
      }
    }
    fprintf(httpd->accessLog, "\"%s\" \"%s\"\n",
	    thd->request.referer == NULL ? "-" : thd->request.referer,
	    thd->request.user_agent == NULL ? "-" : thd->request.user_agent);
  }
  /* Client Error: 4xx	*/
  /* Server Error: 5xx	*/
  else {
    if (thd->request.x_forwarded_for != NULL &&
	strncasecmp(thd->request.x_forwarded_for, "unknown", 7) == 0) {
      wvSyslog(LOG_WARNING, "(%s) \"%s %s %s\" "
		"request forwarded from unknown host.\n",
		thd->client.name == NULL ? thd->client.ip : thd->client.name,
		thd->method.name,
		thd->requestURI,
		thd->http.protocol);
    }
    else if (thd->keepAlive.state != KA_CONTINUE &&
	     (thd->method.name == NULL || thd->requestURI[0] == EOL)) {
      wvSyslog(LOG_EMERG, "attacked or scanned from %s; %s (%d)\n",
		thd->client.name == NULL ? thd->client.ip : thd->client.name,
		getResponsePhrase(thd->rsCode),
		getResponseCode(thd->rsCode));
    }
    else {
      wvSyslog(LOG_ERR, "%s \"%s %s %s\" %s (%d)\n",
		thd->client.name == NULL ? thd->client.ip : thd->client.name,
		thd->method.name,
		thd->requestURI,
		thd->http.protocol,
		getResponsePhrase(thd->rsCode),
		getResponseCode(thd->rsCode));
    }
  }
}

