/*****************************************************************************
 *
 * FILE:	mod_newdir.c
 * DESCRIPTION:	directory listing (available .HEADER/.FOOTER) module for wyvern
 * DATE:	Thu, Sep 14 2000
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2000-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *  Copyright (C) 2000-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_newdir.c,v 1.21.2.2 2002/01/27 16:10:30 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "wyvern.h"
#include "functions.h"
#include "dir.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/
#define	HEADER_FILE	".HEADER"
#define	FOOTER_FILE	".FOOTER"


/*****************************************************************************
 *
 *	definition of global functions for libdir.a
 *
 *****************************************************************************/
extern void	makeContentOfHTML(Thread *, FILE *, String *, int);


/*****************************************************************************
 *
 *	declaration of global functions for libfform.a
 *
 *****************************************************************************/
extern void	printFileForm(String, mode_t, FILE *);
extern void	printFileDescription(String, FILE *);


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static void		includeFile(String, String, FILE *);


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	_init
 * Description:		use libdir
 * Arguments:		none
 * Return Value:	none
 */
void
_init(void)
{
#if	DEBUG
  fputs("[mod_newdir]: plugged-in\n", stderr);
#endif	/* DEBUG */
  initLibDir(makeContentOfHTML);
}

/*
 * Function Name:	_fini
 * Description:		no more use libdir
 * Arguments:		none
 * Return Value:	none
 */
void
_fini(void)
{
  freeLibDir();
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	makeContentOfHTML
 * Description:		make a dirctory listing HTML file
 * Arguments:	
 * Return Value:	none
 */
void
makeContentOfHTML(thd, fout, dir, dnum)
	register Thread *	thd;
	register FILE *		fout;
	String *		dir;
	int			dnum;
{
  int	i;

  /*
   * make HTML text
   */
  fputs(
    "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\">\n"
    "<html lang=\"ja\">\n"
    "<head>\n"
    "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=EUC-JP\">\n"
    "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n"
    "<style type=\"text/css\">\n"
    "<!--\n"
    ".size_b { text-align: right; color: orange }\n"
    ".size_k { text-align: right; color: green }\n"
    ".size_m { text-align: right; color: red }\n"
    ".size_g { text-align: right; color: purple }\n"
    ".text { color: brown }\n"
    "span { color: #ff0033 }\n"
    "table { width: 100%; background-color: #a0b8c8 }\n"
    "tr.none { background-color: transparent }\n"
    "td.size { text-align: right }\n"
    "tr { background-color: #dcdcdc }\n"
    "td { color: #000033 }\n"
    "-->\n"
    "</style>\n",
    fout);
  fprintf(fout,
	"<title>Directory list of %s</title>\n"
	"</head>\n"
	"<body lang=\"ja\">\n"
	"<h1>Directory list of <span>%s</span></h1>\n",
	thd->requestURI, thd->requestURI);

  /*
   * includes '.HEADER' file if it exists.
   */
  includeFile(thd->path, HEADER_FILE, fout);


  /*
   * directory listing
   */
  fprintf(fout,
	"<div>\n"
	"<table border=\"0\" cellspacing=\"0\" summary=\"%s\">\n"
	"<thead>"
	"<tr class=\"none\"><th colspan=\"2\">Name</th>"
	"<th>Last modified</th>"
	"<th>Size</th>"
	"<th>Description</th></tr>\n"
	"<tr><td colspan=\"5\"><hr></tr></thead>\n",
	thd->requestURI);
  for (i = 0; i < dnum; i++) {
    String	path;
    struct stat	sbuf;
    char	tbuf[TINY_BUFSIZE];

    if (strcmp(dir[i], HEADER_FILE) == 0 ||
	strcmp(dir[i], FOOTER_FILE) == 0) {
      continue;	/* don't show HEADER and FOOTER file */
    }

    if (xasprintf(&path, "%s/%s", thd->path, dir[i]) == -1) {
      wvSyslog(LOG_ERR, ERR_OUT_OF_MEMORY);
      continue;	/* buffer overflow ... */
    }
    if (lstat(path, &sbuf) == -1) {
      fprintf(fout,
	"<tr>\n<td></td>"
	"<td><a href=\"%s\">%s</a></td>\n"
	"<td colspan=\"2\">\n",
	dir[i], dir[i]);
      fputs("<td class=\"text\">&nbsp;", fout);
      printFileDescription(dir[i], fout);
      fputs("</td>\n</tr>\n", fout);
      XFree(path);
      continue;
    }
    XFree(path);

    /* print an icon of the file type */
    fputs("<tr>\n<td>", fout);
    printFileForm(dir[i], (mode_t)sbuf.st_mode, fout);
    fputs("</td>\n", fout);

    /* print file name (directory link is a special) */
    if (S_ISDIR(sbuf.st_mode)) {
      fprintf(fout, "<td><a href=\"%s/\">%s</a>/</td>\n", dir[i], dir[i]);
    }
    else {
      fprintf(fout, "<td><a href=\"%s\">%s</a></td>\n", dir[i], dir[i]);
    }

    /* print last modified */
    memset(tbuf, 0, sizeof(tbuf));
    strftime(tbuf, sizeof(tbuf),
	     "%d-%b-%Y&nbsp;%H:%M", localtime(&sbuf.st_mtime));
    fprintf(fout, "<td>&nbsp;%s&nbsp;</td>\n", tbuf);

    /* print file size and file description */
    if (S_ISDIR(sbuf.st_mode)) {
      fputs("<td class=\"size\">&nbsp;-&nbsp;</td>\n"
	    "<td>&nbsp;Directory</td>\n</tr>\n", fout);
    }
    else {
      fputs("<td class=\"", fout);
      if (sbuf.st_size < 1024) {
	fprintf(fout, "size_b\">%ldB", (long)sbuf.st_size);
      }
      else if (sbuf.st_size < 1024 * 1024) {
	fprintf(fout, "size_k\">%ld.%ldK",
		(long)sbuf.st_size / U_KILO,
		((long)sbuf.st_size % U_KILO) * 10 / U_KILO);
      }
      else if (sbuf.st_size < 1024 * 1024 * 1024) {
	fprintf(fout, "size_m\">%ld.%ldM",
		(long)sbuf.st_size / U_MEGA,
		((long)sbuf.st_size % U_MEGA) * 10 / U_MEGA);
      }
      else {
	fprintf(fout, "size_g\">%ld.%ldG",
		(long)sbuf.st_size / U_GIGA,
		((long)sbuf.st_size % U_GIGA) * 10 / U_GIGA);
      }
      fputs("</td>\n<td class=\"text\">&nbsp;", fout);
      printFileDescription(dir[i], fout);
      fputs("</td>\n</tr>\n", fout);
    }
    xfree((String)dir[i]);
  }
  fputs("<tr><td colspan=\"5\"><hr></tr>\n"
	"</table>\n"
	"</div>\n",
	fout);

  /*
   * includes '.FOOTER' file if it exists.
   */
  includeFile(thd->path, FOOTER_FILE, fout);

  fprintf(fout,
	"<address>\n"
	"<a href=\"%s\">%s</a> Server at %s Port %d\n"
	"</address>\n"
	"</body>\n"
	"</html>\n",
	SERVER_ADDRESS, SERVER_VERSION, httpd->vhost->serverName, httpd->port);
}

/*
 * Function Name:	includeFile
 * Description:		include 'HEADER'/'FOOTER' file if it exists
 * Arguments:		file path, file name, outut file pointer
 * Return Value:	none
 */
static void
includeFile(directory, filename, fout)
	String	directory;
	String	filename;
	FILE *	fout;
{
  register FILE *	fin;
  String		path;
  char			buf[BUF_SIZE];

  if (xasprintf(&path, "%s/%s", directory, filename) == -1) {
    wvSyslog(LOG_ERR, ERR_OUT_OF_MEMORY);
    return;
  }

  fin = fopen(path, "r");
  XFree(path);
  if (fin == NULL) {
    return;
  }

  memset(buf, 0, sizeof(buf));
  while (fgets(buf, sizeof(buf), fin) != NULL) {
    fputs(buf, fout);
    memset(buf, 0, sizeof(buf));
  }
  fputs("\n<hr>\n", fout);

  fclose(fin);
}

