/*****************************************************************************
 *
 * FILE:	mod_newlog.c
 * DESCRIPTION:	wyvern logging module with DoS protection
 * DATE:	Wed, Sep 20 2000
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2000-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *  Copyright (C) 2000-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_newlog.c,v 1.15.2.2 2002/02/19 11:50:12 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <time.h>
#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	declaration of global functions
 *
 *****************************************************************************/
extern void	writeMoreLogs(Thread *);


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/
#ifndef	HAVE_SETLINEBUF
#define	setlinebuf(x)	setvbuf((x), (char *)NULL, _IOLBF, 0)
#endif	/* HAVE_SETLINEBUF */

#define	DOS_INTERVAL_SEC	5	/* second */
#define	DOS_INTERVAL_MSEC	1000000	/* microsecond (but not used...) */
#define	DOS_ATTACK_BOUND	10	/* # of DoS attack per interval sec. */

typedef struct _DoS {	/* DoS attack detection structure */
  String		clientIP; /* client's IP address */
  rsCode_t		rsCode;	/* response code */
  struct timeval	atime;	/* last access time */
  unsigned int		count;	/* access count */
  unsigned int		total;	/* total access count */
} DoS;


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/
static DoS	dos;	/* to detective DoS attack */


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static void	flushLogs(Thread *);


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	_init
 * Description:		initialize DoS structure (automatic loading)
 * Arguments:		none
 * Return Value:	none
 */
void
_init(void)
{
#if	DEBUG
  fputs("[mod_newlog]: plugged-in\n", stderr);
#endif	/* DEBUG */
  memset(&dos, 0, sizeof(DoS));
}


/*
 * Function Name:	writeMoreLogs
 * Description:		write the log message in log file
 * Arguments:		thread
 * Return Value:	none
 */
void
writeMoreLogs(thd)
	register Thread *	thd;
{
  struct timeval	now;

  gettimeofday(&now, NULL);

  if (dos.clientIP != NULL && strcmp(thd->client.ip, dos.clientIP) == 0 &&
      thd->rsCode == dos.rsCode &&
      now.tv_sec - dos.atime.tv_sec < DOS_INTERVAL_SEC) {
    /* squeeze error log */
    dos.count++;
#if	DEBUG
    fprintf(stderr, "[mod_newlog]: %s attacks %d counts\n",
	    dos.clientIP, dos.count);
#endif	/* DEBUG */
    if (dos.count > DOS_ATTACK_BOUND) {
#if	0
      setResponseCode(thd, CODE_FORBIDDEN);
      flushLogs(thd);	/* flush log */
#endif
#if	DEBUG
      fputs("[mod_newlog]: actual DoS attacks!\n", stderr);
      fprintf(stderr, "[mod_newlog] DoS(%d) to %s\n", dos.count, thd->path);
#endif	/* DEBUG */
      return;
    }
  }
  else {
    dos.total = dos.count;
    dos.count = 0;
  }

  flushLogs(thd);	/* flush log */
  XFree(dos.clientIP);
  dos.clientIP	= xstrdup(thd->client.ip);
  dos.atime	= now;
  dos.rsCode	= thd->rsCode;
}

/*
 * Function Name:	flushLogs
 * Description:		write the log message in log file
 * Arguments:		thread
 * Return Value:	none
 */
static void
flushLogs(thd)
	register Thread *	thd;
{
  if (dos.count > 0 && dos.rsCode >= CODE_BAD_REQUEST) {
    return;
  }

  if (dos.total > 1 && dos.rsCode >= CODE_BAD_REQUEST) {
    wvSyslog(LOG_INFO, "last message repeated %d times\n", dos.total);
    return;
  }

  /* Informational: 1xx	*/
  /* Successful:    2xx	*/
  /* Redirection:   3xx	*/
  if (thd->rsCode < CODE_BAD_REQUEST) {
    if (thd->request.x_forwarded_for != NULL) {
      fprintf(httpd->accessLog, "%s(%s) ",
	      thd->client.name == NULL ? thd->client.ip : thd->client.name,
	      thd->request.x_forwarded_for);

    }
    else {
      fprintf(httpd->accessLog, "%s ",
	      thd->client.name == NULL ? thd->client.ip : thd->client.name);
    }
    fprintf(httpd->accessLog, "%s %s [%s] \"%s %s %s\" %d ",
		thd->identUser == NULL ? "-" : thd->identUser,
		thd->authUser == NULL ? "-" : thd->authUser,
		getTimeOfRFC822(thd->atime),
		thd->method.name,
		thd->requestURI,
		thd->http.protocol,
		getResponseCode(thd->rsCode));
    if (thd->rsCode > CODE_PARTIAL_CONTENT) {	/* for Redirection: 3xx */
      fputs("- ", httpd->accessLog);
    }
    else {
      if (thd->method.type == M_HEAD || thd->method.type == M_GET) {
#if	HAVE_ZLIB_H
	fprintf(httpd->accessLog, "%ld (%4.1f%%) ",
		(long)thd->file.st_size, thd->gzippct);
#else
	fprintf(httpd->accessLog, "%ld ", (long)thd->file.st_size);
#endif	/* HAVE_ZLIB_H */
      }
      else if (thd->method.type == M_POST) {
	fprintf(httpd->accessLog, "%d ", thd->request.content_length);
      }
      else {
	fputs("- ", httpd->accessLog);
      }
    }
    fprintf(httpd->accessLog, "\"%s\" \"%s\"\n",
	    thd->request.referer == NULL ? "-" : thd->request.referer,
	    thd->request.user_agent == NULL ? "-" : thd->request.user_agent);
  }
  /* Client Error: 4xx	*/
  /* Server Error: 5xx	*/
  else {
    if (thd->request.x_forwarded_for != NULL &&
	strncasecmp(thd->request.x_forwarded_for, "unknown", 7) == 0) {
      wvSyslog(LOG_WARNING,
		"(%s) \"%s %s %s\" request forwarded from unknown host.\n",
		thd->client.name == NULL ? thd->client.ip : thd->client.name,
		thd->method.name,
		thd->requestURI,
		thd->http.protocol);
    }
    else if (thd->keepAlive.state != KA_CONTINUE &&
	     (thd->method.name == NULL || thd->requestURI[0] == EOL)) {
      wvSyslog(LOG_EMERG, "attacked or scanned from %s; %s (%d)\n",
		thd->client.name == NULL ? thd->client.ip : thd->client.name,
		getResponsePhrase(thd->rsCode),
		getResponseCode(thd->rsCode));
    }
    else {
      wvSyslog(LOG_ERR, "%s \"%s %s %s\" %s (%d)\n",
		 thd->client.name == NULL ? thd->client.ip : thd->client.name,
		 thd->method.name,
		 thd->requestURI,
		 thd->http.protocol,
		 getResponsePhrase(thd->rsCode),
		 getResponseCode(thd->rsCode));
    }
  }
}

