/*****************************************************************************
 *
 * FILE:	signal.c
 * DESCRIPTION:	signal handler
 * DATE:	Mon, Nov 15 1999
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 1999-2002 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *   Copyright (C) 1999-2002 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *   All rights reserved.
 *  
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in
 *      the documentation and/or other materials provided with the
 *      distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY KOUICHI ABE (WALL) ``AS IS'' AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KOUICHI ABE (WALL) BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *   WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *   OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 *   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: signal.c,v 1.20.2.3 2002/02/05 12:16:40 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <time.h>
#include <signal.h>
#if	SIGTSTP
#include <sys/file.h>
#include <sys/ioctl.h>
#endif	/* SIGTSTP */
#if	HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif	/* HAVE_SYS_WAIT_H */
#include "wyvern.h"
#include "functions.h"


/*****************************************************************************
 *
 *	declaration of local functions
 *
 *****************************************************************************/
static void	trapSIGHUP(void);
#if	0
static void	trapSIGXFSZ(void);
#endif
static void	trap(int);
static void	waitChild(void);
#if	DEBUG
static void	debugSignal(void);
#endif	/* DEBUG */


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/
static String	signame[] = {	/* cf. /usr/include/sys/signal.h */
  "NOSIGNAL",	/* dummy */
  "SIGHUP",	/* hangup */
  "SIGINT",	/* interrupt */
  "SIGQUIT",	/* quit */
  "SIGILL",	/* illegal instruction (not reset when caught) */
#ifndef _POSIX_SOURCE
  "SIGTRAP",	/* trace trap (not reset when caught) */
#endif
  "SIGABRT",	/* abort() : SIGIOT compatibility */
#ifndef _POSIX_SOURCE
  "SIGEMT",	/* EMT instruction */
#endif
  "SIGFPE",	/* floating point exception */
  "SIGKILL",	/* kill (cannot be caught or ignored) */
#ifndef _POSIX_SOURCE
  "SIGBUS",	/* bus error */
#endif
  "SIGSEGV",	/* segmentation violation */
#ifndef _POSIX_SOURCE
  "SIGSYS",	/* non-existent system call invoked */
#endif
  "SIGPIPE",	/* write on a pipe with no one to read it */
  "SIGALRM",	/* alarm clock */
  "SIGTERM",	/* software termination signal from kill */
#ifndef _POSIX_SOURCE
  "SIGURG",	/* urgent condition on IO channel */
#endif
  "SIGSTOP",	/* sendable stop signal not from tty */
  "SIGTSTP",	/* stop signal from tty */
  "SIGCONT",	/* continue a stopped process */
  "SIGCHLD",	/* to parent on child stop or exit */
  "SIGTTIN",	/* to readers pgrp upon background tty read */
  "SIGTTOU",	/* like TTIN for output if (tp->t_local&LTOSTOP) */
#ifndef _POSIX_SOURCE
  "SIGIO",	/* input/output possible signal */
  "SIGXCPU",	/* exceeded CPU time limit */
  "SIGXFSZ",	/* exceeded file size limit */
  "SIGVTALRM",	/* virtual time alarm */
  "SIGPROF",	/* profiling time alarm */
  "SIGWINCH",	/* window size changes */
  "SIGINFO",	/* information request */
#endif
  "SIGUSR1",	/* user defined signal 1 */
  "SIGUSR2",	/* user defined signal 2 */
};


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * Function Name:	setSignalTrap
 * Description:		set the signal handler
 * Arguments:		none
 * Return Value:	none
 */
void
setSignalTrap(void)
{
  struct sigaction	act;

  /* quit */
  sigemptyset(&act.sa_mask);
  act.sa_flags	 = 0;
  act.sa_handler = (void *)trap;
  sigaction(SIGINT,  &act, NULL);
  sigaction(SIGQUIT, &act, NULL);
  sigaction(SIGTERM, &act, NULL);
  /* bug */
  sigaction(SIGBUS,  &act, NULL);
  sigaction(SIGSEGV, &act, NULL);

  /* cgi/ssi */
  act.sa_flags	 = SA_NOCLDSTOP;
  act.sa_handler = (void *)waitChild;
  sigaction(SIGCHLD, &act, NULL);

  /* ignore */
#if	DEBUG
  act.sa_handler = (void *)debugSignal;
  sigaction(SIGPIPE, &act, NULL);
#else
  act.sa_handler = SIG_IGN;
  sigaction(SIGPIPE, &act, NULL);
#endif	/* DEBUG */

  /* restart */
  act.sa_handler = (void *)trapSIGHUP;
  sigaction(SIGHUP,  &act, NULL);
}


/*
 * Function Name:	trapSIGHUP
 * Description:		trap SIGHUP signals and set reboot flag
 * Arguments:		none
 * Return Value:	none
 */
static void
trapSIGHUP(void)
{
  httpd->reboot = TRUE;
}

#if	0
/*
 * Function Name:	trapSIGXFSZ
 * Description:		trap SIGXFSZ signals
 * Arguments:		none
 * Return Value:	none
 */
static void
trapSIGXFSZ(void)
{
  wvSyslog(LOG_ALERT, "request creates a file exceeded %dB\n", MAX_FILE_SIZE);
}
#endif

/*
 * Function Name:	trap
 * Description:		trap all signals
 * Arguments:		signal
 * Return Value:	never return
 */
static void
trap(sig)
	int	sig;
{
  switch (sig) {
    case SIGINT:
    case SIGQUIT:
    case SIGTERM:
      wvSyslog(LOG_NOTICE, "terminated by %s\n", signame[sig]);
      break;
    case SIGBUS:
    case SIGSEGV:
      wvSyslog(LOG_CRIT, "killed by %s ... it's a BUG!\n", signame[sig]);
      break;
    default:
      wvSyslog(LOG_ALERT, "killed by unexpected signal (%s) ... BUG?\n",
	       signame[sig]);
      break;
  }
  quit();
}


/*****************************************************************************
 *****************************************************************************/

/*
 * Function Name:	waitChild
 * Description:		wait for child process termination
 * Arguments:		none
 * Return Value:	none
 */
static void
waitChild(void)
{
  sigset_t	set;
  sigset_t	oset;
  pid_t		pid;
  int		status;

  sigprocmask(SIG_SETMASK, NULL, &set);	/* get current signal mask */
  sigaddset(&set, SIGCHLD);		/* add SIGCHLD */
  sigprocmask(SIG_BLOCK, &set, &oset);	/* block SIGCHLD */

  while ((pid = waitpid((pid_t)0, &status, WNOHANG)) > 0) {
#if	DEBUG
    fprintf(stderr, "[signal] pid=%d, status=%d\n", pid, WEXITSTATUS(status));
#endif	/* DEBUG */
  }

  sigprocmask(SIG_SETMASK, &oset, NULL);	/* reset signal mask */
}

#if	DEBUG
/*
 * Function Name:	debugSignal
 * Description:		debug function for signal handler
 * Arguments:		none
 * Return Value:	none
 */
static void
debugSignal(void)
{
  fputs("[signal]: caught a signal (PIPE)\n", stderr);
}
#endif	/* DEBUG */

